@@ -813,6 +813,16 @@ sub MANAGE_create {
     }
 }
 
+sub MANAGE_use {
+    my $mc = shift->parse(qr/^use (\w+)$/,
+                          "usage: USE <service_or_pool_name>");
+    my ($name) = $mc->args;
+    return $mc->err("Non-existent pool or service '$name'") unless $pool{$name} || $service{$name};
+
+    $mc->{ctx}{last_created} = $name;
+    return $mc->ok;
+}
+
 sub MANAGE_pool {
     my $mc = shift->parse(qr/^pool (\w+) (\w+) (\d+.\d+.\d+.\d+)(?::(\d+))?$/);
     my ($cmd, $name, $ip, $port) = $mc->args;
