@@ -220,6 +220,8 @@ sub event_read {
     my $hd = $self->read_request_headers;
     return unless $hd;
 
+    return if $self->{service}->run_hook('start_http_request', $self);
+
     # we must stop watching for events now, otherwise if there's
     # PUT/POST overflow, it'll be sent to ClientHTTPBase, which can't
     # handle it yet.  must wait for the selector (which has as much
@@ -302,6 +304,9 @@ sub _serve_request {
 
     my $uri = Perlbal::Util::durl($self->{replacement_uri} || $hd->request_uri);
 
+    # chop off the query string
+    $uri =~ s/\?.*//;
+
     # don't allow directory traversal
     if ($uri =~ /\.\./ || $uri !~ m!^/!) {
         return $self->_simple_response(403, "Bogus URL");
