@@ -44,7 +44,7 @@ use fields (
                                  # connections to activate pressure relief at
             'queue_relief_chance', # int:0-100; % chance to take a standard priority
                                    # request when we're in pressure relief mode
-            'trusted_upstreams', # Net::Netmask object containing netmasks for trusted upstreams
+            'trusted_upstream_proxies', # Net::Netmask object containing netmasks for trusted upstreams
             'always_trusted', # bool; if true, always trust upstreams
 
             # Internal state:
@@ -275,7 +275,7 @@ our $tunables = {
                 return 0;
             }
 
-            return 1 if $self->{trusted_upstreams} = Net::Netmask->new2($val);
+            return 1 if $self->{trusted_upstream_proxies} = Net::Netmask->new2($val);
             $$errref = "Error defining trusted upstream proxies: " . Net::Netmask::errstr();
             return 0;
         },
@@ -327,7 +327,7 @@ our $tunables = {
             my ($self, $val, $errref) = @_;
             #FIXME: require absolute paths?
             return 1 if $val && -d $val;
-            $$errref = "Directory not found for service $self->{name} (buffer_uploads_path)";
+            $$errref = "Directory ($val) not found for service $self->{name} (buffer_uploads_path)";
             return 0;
         },
     },
@@ -420,7 +420,7 @@ sub new {
     $self->{buffer_uploads_path} = undef;
 
     # don't have an object for this yet
-    $self->{trusted_upstreams} = undef;
+    $self->{trusted_upstream_proxies} = undef;
 
     # bare data structure for extra header info
     $self->{extra_headers} = { remove => [], insert => [] };
@@ -585,8 +585,9 @@ sub set {
     return $mc->err("Unknown service parameter '$key'");
 }
 
-# run the hooks in a list one by one until one hook returns 1.  returns
-# 1 or 0 depending on if any hooks handled the request.
+# run the hooks in a list one by one until one hook returns a true
+# value.  returns 1 or 0 depending on if any hooks handled the
+# request.
 sub run_hook {
     my Perlbal::Service $self = shift;
     my $hook = shift;
@@ -594,7 +595,7 @@ sub run_hook {
         # call all the hooks until one returns true
         foreach my $hookref (@$ref) {
             my $rval = $hookref->[1]->(@_);
-            return 1 if defined $rval && $rval;
+            return 1 if $rval;
         }
     }
     return 0;
@@ -815,14 +816,12 @@ sub register_boredom {
     # now try to fetch a client for it
     my Perlbal::ClientProxy $cp = $self->get_client;
     if ($cp) {
-        if ($be->assign_client($cp)) {
-            return;
-        } else {
-            # don't want to lose client, so we (unfortunately)
-            # stick it at the end of the waiting queue.
-            # fortunately, assign_client shouldn't ever fail.
-            $self->request_backend_connection($cp);
-        }
+        return if $be->assign_client($cp);
+
+        # don't want to lose client, so we (unfortunately)
+        # stick it at the end of the waiting queue.
+        # fortunately, assign_client shouldn't ever fail.
+        $self->request_backend_connection($cp);
     }
 
     # don't hang onto more bored, persistent connections than
@@ -872,11 +871,13 @@ sub note_bad_backend_connect {
     $self->spawn_backends;
 }
 
-sub request_backend_connection {
+sub request_backend_connection { # : void
     my Perlbal::Service $self;
     my Perlbal::ClientProxy $cp;
     ($self, $cp) = @_;
 
+    return unless $cp && ! $cp->{closed};
+
     my $hi_pri = 0;  # by default, low priority
 
     # is there a defined high-priority cookie?
@@ -927,6 +928,9 @@ sub request_backend_connection {
             $self->spawn_backends;
             return;
         }
+
+        # assign client can end up closing the connection, so check for that
+        return if $cp->{closed};
     }
 
     if ($hi_pri) {
@@ -1024,7 +1028,7 @@ sub trusted_ip {
 
     return 1 if $self->{'always_trusted'};
 
-    my $tmap = $self->{trusted_upstreams};
+    my $tmap = $self->{trusted_upstream_proxies};
     return 0 unless $tmap;
 
     # try to use it as a Net::Netmask object
